/*****************************************************************************/
/* Copyright (c) 2008 NXP B.V.                                               */
/*                                                                           */
/* This program is free software; you can redistribute it and/or modify      */
/* it under the terms of the GNU General Public License as published by      */
/* the Free Software Foundation, using version 2 of the License.             */
/*                                                                           */
/* This program is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the              */
/* GNU General Public License for more details.                              */
/*                                                                           */
/* You should have received a copy of the GNU General Public License         */
/* along with this program; if not, write to the Free Software               */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307       */
/* USA.                                                                      */
/*                                                                           */
/*****************************************************************************/
/**
    @file   xml.c
    @brief  XML utilities

@b  Application:  tmmanifest
*//*

Rev Date        Author      Comments
--------------------------------------------------------------------------------
  1 20080519    batelaan    Initial

--------------------------------------------------------------------------------
    For consistency and standardisation retain the Section Separators.
*/

/***********************************************
* INCLUDE FILES                                *
************************************************/

#include "xml.h"
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

/***********************************************
* LOCAL MACROS                                 *
* recommendation only: <MODULE><_WORD>+        *
************************************************/

/******************************************************************
* LOCAL TYPEDEFS                                                  *
* recommendation only:                [p]phStbTemplate_<Word>+_t  *
*******************************************************************/

/******************************************************************
* STATIC FUNCTION PROTOTYPES                  <Module>_<Word>+    *
*******************************************************************/

/******************************************************************
* STATIC DATA                  g[k|p|kp|pk|kpk]<Module>_<Word>+   *
*******************************************************************/

/**************************************************************************
* EXPORTED DATA                     g[k|p|kp|pk|kpk]phStbTemplate_<Word>+ *
***************************************************************************/

FILE * gpxmlFilePtr;
bool gxmlTagNeedClosing = false;
unsigned int indentLevel;

/*******************************************************************************
* FUNCTION IMPLEMENTATION  <Module>[_<Word>+] for static functions             *
*                          ph[<layer>]<Module>[_<Word>+] for exported functions*
********************************************************************************/

/******************************************************************************/
void xmlSetOutput(FILE *fp)
{
    gpxmlFilePtr = fp;
}

/******************************************************************************/
void xmlIndent(int count)
{
    indentLevel += count;
    unsigned int n;
    fputc('\n', gpxmlFilePtr);
    for (n = indentLevel; n > 0; n--)
    {
        fputc(' ', gpxmlFilePtr);
    }
}

/******************************************************************************/
void xmlCheckState(void)
{
    if (gxmlTagNeedClosing) {
        fputc('>', gpxmlFilePtr);
        gxmlTagNeedClosing = false;
    }
}

/******************************************************************************/
void xmlStartTag(const char *tag)
{
    xmlCheckState();
    xmlIndent(1);
    fprintf(gpxmlFilePtr, "<%s", tag);
    gxmlTagNeedClosing = true;
}

/******************************************************************************/
void xmlEndTag(const char *tag)
{
    xmlCheckState();
    fprintf(gpxmlFilePtr, "</%s>", tag);
    xmlIndent(-1);
}

/******************************************************************************/
void xmlAttr(const char *name, const char * format, ...)
{
    char value[MAX_XML_ATTR_VALUE_LEN + 2];
    va_list ap;
    va_start(ap, format);
    int numChars = vsnprintf(value, MAX_XML_ATTR_VALUE_LEN + 1, format, ap);
    va_end(ap);
    if (numChars > MAX_XML_ATTR_VALUE_LEN) {
        fprintf(stderr, "**FATAL: xmlAttr(\"%s\", \"%s\", ...): value too long (> %d chars)!\n"
                "\tAttribute value was: \"%s\"\n",
                name, format, MAX_XML_ATTR_VALUE_LEN, value);
        abort();
    }

    if (!gxmlTagNeedClosing) {
        fprintf(stderr, "**FATAL: xmlAttr(\"%s\", \"%s\", ...) called when not inside a XML tag.\n"
                "\tAttribute value was: \"%s\"\n",
                name, format, value);
        abort();
    }
    fputc(' ', gpxmlFilePtr);
    fputs(name, gpxmlFilePtr);
    fputc('=', gpxmlFilePtr);
    bool containsSQoute = index(value, '\'') != NULL;
    bool containsDQoute = index(value, '"') != NULL;
    if (containsSQoute && containsDQoute)
    {
        fprintf(stderr, "**FATAL: xmlAttr(\"%s\", \"%s\", ...): value has single AND double quotes!\n"
                "\tAttribute value was: \"%s\"\n",
                name, format, value);
        abort();
    }
    const char quote = containsDQoute ? '\'' : '"';
    fputc(quote, gpxmlFilePtr);
    fputs(value, gpxmlFilePtr);
    fputc(quote, gpxmlFilePtr);
}

/******************************************************************************/
void xmlText(const char *text)
{
    const char *cp;
    for (cp = text; *cp; cp++)
    {
        switch (*cp)
        {
            case '&': {
                fputs("&amp;", gpxmlFilePtr);
                break;
            }
            case '<': {
                fputs("&lt;", gpxmlFilePtr);
                break;
            }
            case '>': {
                fputs("&gt;", gpxmlFilePtr);
                break;
            }
            default: {
                fputc(*cp, gpxmlFilePtr);
                break;
            }
        }
    }
}
/******************************************************************************/
