#############################################################
#
# directfb
#
#############################################################
DIRECTFB_SITE:=http://www.directfb.org/downloads/Core
DIRECTFB_CAT:=zcat

DIRECTFB_VERSION=$(strip $(subst ",, $(BR2_DIRECTFB_VERSION)))
ifeq "$(DIRECTFB_VERSION)" "1.0.0"
DIRECTFB_SOURCE:=DirectFB-$(DIRECTFB_VERSION)_nxp.tar.gz
DIRECTFB_DIR:=$(BUILD_DIR)/DirectFB-$(DIRECTFB_VERSION)_nxp
FUSION_VERSION:=3.2.3
else
DIRECTFB_SOURCE:=DirectFB-$(DIRECTFB_VERSION).tar.gz
DIRECTFB_DIR:=$(BUILD_DIR)/DirectFB-$(DIRECTFB_VERSION)
FUSION_VERSION:=8.0.0
endif

DIRECTFB_CFLAGS:=$(TARGET_CFLAGS) -I. -I$(DIRECTFB_DIR)/include/fusion -I$(STAGING_DIR)/usr/include -I$(STAGING_DIR)/include

# TRACING_AND_DEBUG_ENABLE:= --enable-debug --enable-trace

# What if we need large file support.
ifeq ($(BR2_LARGEFILE),y)
DIRECTFB_CFLAGS+=-D_LARGEFILE_SOURCE -D_LARGEFILE64_SOURCE -D_FILE_OFFSET_BITS=64
endif

# If we are building multi with the fusion kernel module then enable-multi and specify version.
ifeq ($(BR2_PACKAGE_DIRECTFB_MULTI),y)
DIRECTFB_MULTI:=--enable-multi
else
DIRECTFB_MULTI:=
endif

#If we have enabled freetype then build with support for it.
ifeq ($(BR2_PACKAGE_FREETYPE),y)
DIRECTFB_FREETYPE:=--enable-freetype
else
DIRECTFB_FREETYPE:=
endif

#If we have enabled sysfs then build with support for it
ifeq ($(BR2_PACKAGE_LIBSYSFS),y)
DIRECTFB_SYSFS:=--enable-sysfs
else
DIRECTFB_SYSFS:=
endif

#If we have enabled zlib then build with support for it
ifeq ($(BR2_PACKAGE_ZLIB),y)
DIRECTFB_ZLIB:=--enable-zlib
else
DIRECTFB_ZLIB:=
endif

# If no X server specified then disable X
ifeq ($(BR2_PACKAGE_XSERVER_none),y)
DIRECTFB_X:=--disable-x11
else
DIRECTFB_X:=--enable-x11
endif

#For DirectFB 1.0.X we use fbdev for 1.2.X we use devmem
ifeq "$(DIRECTFB_VERSION)" "1.0.0"
DIRECTFB_SYSTEM= \
	--enable-devmem=no \
	--enable-fbdev=yes
else
DIRECTFB_SYSTEM= \
	--enable-devmem=yes \
	--enable-fbdev=no
endif

# For DirectFB 1.0 use our image providers.
ifeq "$(DIRECTFB_VERSION)" "1.0.0"
DIRECTFB_IMAGEPROVIDERS = \
 	--disable-jpeg \
	--disable-png \
	--disable-gif
else
# For DirectFB 1.2.x we also use our own Image Provider.
DIRECTFB_IMAGEPROVIDERS = \
 	--disable-jpeg \
	--disable-png \
	--disable-gif
endif

# Do you want debug enabled or not?
ifeq ($(BR2_PACKAGE_DIRECTFB_DEBUG),y)
DIRECTFB_DEBUG = \
 	--enable-debug
else
DIRECTFB_DEBUG = \
 	--disable-debug
endif

$(DL_DIR)/$(DIRECTFB_SOURCE):
	$(WGET) -P $(DL_DIR) $(DIRECTFB_SITE)/$(DIRECTFB_SOURCE)

directfb-source: $(DL_DIR)/$(DIRECTFB_SOURCE)

$(DIRECTFB_DIR)/.unpacked: $(DL_DIR)/$(DIRECTFB_SOURCE)
	$(DIRECTFB_CAT) $(DL_DIR)/$(DIRECTFB_SOURCE) | tar -C $(BUILD_DIR) $(TAR_OPTIONS) -
	touch $(DIRECTFB_DIR)/.unpacked

$(DIRECTFB_DIR)/.patched:$(DIRECTFB_DIR)/.unpacked
ifeq ($(BR2_PACKAGE_LINUX_FUSION),y)
	mkdir -p $(DIRECTFB_DIR)/include/fusion/linux
	tar zxfO $(DL_DIR)/linux-fusion-$(FUSION_VERSION).tar.gz \
		linux-fusion-$(FUSION_VERSION)/linux/include/linux/fusion.h > $(DIRECTFB_DIR)/include/fusion/linux/fusion.h
endif
	cp -R package/directfb/overlay$(DIRECTFB_VERSION)/* $(DIRECTFB_DIR)
	toolchain/patch-kernel.sh $(DIRECTFB_DIR) package/directfb/ libdir-la.patch DirectFB-$(DIRECTFB_VERSION)*.patch
	touch  $(DIRECTFB_DIR)/.patched

$(DIRECTFB_DIR)/.configured: $(DIRECTFB_DIR)/.patched
	echo DFB TARGET_LDFLAGS=$(TARGET_LDFLAGS)
	(cd $(DIRECTFB_DIR); rm -f config.cache; \
        ac_cv_header_linux_wm97xx_h=no \
	ac_cv_header_linux_sisfb_h=no \
	$(AUTO_CONFIGURE_TARGET) \
	--prefix=/usr \
	--sysconfdir=/etc \
	--localstatedir=/var \
	--with-gfxdrivers=none \
	--with-inputdrivers=linuxinput \
	--disable-sdl \
	--disable-video4linux \
	--disable-video4linux2 \
	--with-smooth-scaling \
	$(DIRECTFB_DEBUG) \
	$(DIRECTFB_IMAGEPROVIDERS) \
	$(DIRECTFB_SYSTEM) \
	$(DIRECTFB_ZLIB) \
	$(DIRECTFB_FREETYPE) \
	$(DIRECTFB_SYSFS) \
	$(DIRECTFB_X) \
	$(DIRECTFB_MULTI) \
	$(TRACING_AND_DEBUG_ENABLE) \
	);
	touch  $(DIRECTFB_DIR)/.configured

$(DIRECTFB_DIR)/.compiled: $(DIRECTFB_DIR)/.configured
	$(MAKE) -C $(DIRECTFB_DIR)
	touch $(DIRECTFB_DIR)/.compiled

$(STAGING_DIR)/usr/lib/libdirectfb.so: $(DIRECTFB_DIR)/.compiled
	$(MAKE) DESTDIR=$(STAGING_DIR) -C $(DIRECTFB_DIR) install
	$(SED) "/^libdir=/ s,/usr/lib,$(STAGING_DIR)/usr/lib,g" $(STAGING_DIR)/usr/lib/libdirectfb.la
	$(SED) "/^dependency_libs=/ s,/usr/lib,$(STAGING_DIR)/usr/lib,g" $(STAGING_DIR)/usr/lib/libdirectfb.la
	$(SED) "/^libdir=/ s,/usr/lib,$(STAGING_DIR)/usr/lib,g" $(STAGING_DIR)/usr/lib/libdirect.la
	$(SED) "/^dependency_libs=/ s,/usr/lib,$(STAGING_DIR)/usr/lib,g" $(STAGING_DIR)/usr/lib/libdirect.la
	$(SED) "/^libdir=/ s,/usr/lib,$(STAGING_DIR)/usr/lib,g" $(STAGING_DIR)/usr/lib/libfusion.la
	$(SED) "/^dependency_libs=/ s,/usr/lib,$(STAGING_DIR)/usr/lib,g" $(STAGING_DIR)/usr/lib/libfusion.la
	touch -c $(STAGING_DIR)/usr/lib/libdirectfb.so

$(TARGET_DIR)/usr/lib/libdirectfb.so: $(STAGING_DIR)/usr/lib/libdirectfb.so
	mkdir -p $(TARGET_DIR)/usr/lib
	mkdir -p $(TARGET_DIR)/usr/bin
	mkdir -p $(TARGET_DIR)/usr/share/directfb-$(DIRECTFB_VERSION)
	# Copy in the useful DirectFB apps
	cp -dpf $(STAGING_DIR)/usr/bin/$(GNU_TARGET_NAME)-dfbdump $(TARGET_DIR)/usr/bin/dfbdump
	cp -dpf $(STAGING_DIR)/usr/bin/$(GNU_TARGET_NAME)-dfbg $(TARGET_DIR)/usr/bin/dfbg
	cp -dpf $(STAGING_DIR)/usr/bin/$(GNU_TARGET_NAME)-dfbinfo $(TARGET_DIR)/usr/bin/dfbinfo
	cp -dpf $(STAGING_DIR)/usr/bin/$(GNU_TARGET_NAME)-dfblayer $(TARGET_DIR)/usr/bin/dfblayer
	cp -dpf $(STAGING_DIR)/usr/bin/$(GNU_TARGET_NAME)-dfbscreen $(TARGET_DIR)/usr/bin/dfbscreen
	cp -dpf $(STAGING_DIR)/usr/bin/$(GNU_TARGET_NAME)-dfbinput $(TARGET_DIR)/usr/bin/dfbinput
	# Copy in the DirectFB libraries
	cp -dpf $(STAGING_DIR)/usr/lib/libdirect* $(TARGET_DIR)/usr/lib/
	cp -dpf $(STAGING_DIR)/usr/lib/libfusion* $(TARGET_DIR)/usr/lib/
	cp -rdpf $(STAGING_DIR)/usr/lib/directfb-1.* $(TARGET_DIR)/usr/lib/
	# Copy in the DirectFB cursor
	cp -rdpf $(STAGING_DIR)/usr/share/directfb-$(DIRECTFB_VERSION) $(TARGET_DIR)/usr/share/
	# Strip the libraries
	-$(STRIPCMD) $(STRIP_STRIP_UNNEEDED) $(TARGET_DIR)/usr/bin/dfbdump
	-$(STRIPCMD) $(STRIP_STRIP_UNNEEDED) $(TARGET_DIR)/usr/bin/dfbg
	-$(STRIPCMD) $(STRIP_STRIP_UNNEEDED) $(TARGET_DIR)/usr/bin/dfbinfo
	-$(STRIPCMD) $(STRIP_STRIP_UNNEEDED) $(TARGET_DIR)/usr/bin/dfblayer
	-$(STRIPCMD) $(STRIP_STRIP_UNNEEDED) $(TARGET_DIR)/usr/bin/dfbscreen
	-$(STRIPCMD) $(STRIP_STRIP_UNNEEDED) $(TARGET_DIR)/usr/bin/dfbinput
	-$(STRIPCMD) $(STRIP_STRIP_UNNEEDED) $(TARGET_DIR)/usr/lib/libdirect*.so
	-$(STRIPCMD) $(STRIP_STRIP_UNNEEDED) $(TARGET_DIR)/usr/lib/libfusion*.so

directfb: uclibc \
	$(if $(BR2_PACKAGE_FREETYPE), freetype) \
	$(if $(BR2_PACKAGE_ZLIB), zlib) \
	$(if $(BR2_PACKAGE_LIBPNG), libpng) \
	$(if $(BR2_PACKAGE_JPEG), jpeg) \
	$(if $(BR2_PACKAGE_LIBSYSFS), libsysfs) \
	$(TARGET_DIR)/usr/lib/libdirectfb.so

directfb-clean:
	-$(MAKE) DESTDIR=$(TARGET_DIR) CC=$(TARGET_CC) -C $(DIRECTFB_DIR) uninstall
	-$(MAKE) -C $(DIRECTFB_DIR) clean

directfb-dirclean:
	rm -rf $(DIRECTFB_DIR)

#############################################################
#
# Toplevel Makefile options
#
#############################################################
ifeq ($(strip $(BR2_PACKAGE_DIRECTFB)),y)
TARGETS+=directfb
endif
