/*****************************************************************************/
/* Copyright (c) 2009 NXP B.V.                                               */
/*                                                                           */
/* This library is free software; you can redistribute it and/or             */
/* modify it under the terms of the GNU Lesser General Public                */
/* License as published by the Free Software Foundation, using               */
/* version 2.1 of the License.                                               */
/*                                                                           */
/* This library is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU          */
/* Lesser General Public License for more details.                           */
/*                                                                           */
/* You should have received a copy of the GNU Lesser General Public          */
/* License along with this library; if not, write to the Free Software       */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   */
/*                                                                           */
/*****************************************************************************/
/**
    @file   phStbDbg_UnitTest2Linux.c
    @brief  Unit test for phStbDbg on Linux.

    Runs N iterations of DBG_INIT / DBG_REGISTER_MODULE,
    as specified by the command line.

@b  Component:  phStbDbg.

*//*

Rev Date        Author      Comments
--------------------------------------------------------------------------------
  1 20080225    batelaan    Initial
  2 20090120    batelaan    Update file copyright/license text
--------------------------------------------------------------------------------
    For consistency and standardisation retain the Section Separators.
*/

/***********************************************
* INCLUDE FILES                                *
************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <phStbDbg.h>

/***********************************************
* LOCAL MACROS                                 *
* recommendation only: <MODULE><_WORD>+        *
************************************************/

/******************************************************************
* LOCAL TYPEDEFS                                                  *
* recommendation only:                     [p]phStbDbg_<Word>+_t  *
*******************************************************************/

/******************************************************************
* STATIC FUNCTION PROTOTYPES                  <Module>_<Word>+    *
*******************************************************************/

/******************************************************************
* STATIC DATA                  g[k|p|kp|pk|kpk]<Module>_<Word>+   *
*******************************************************************/

DBG_UNIT_STATIC(gDbgUnit);

/******************************************************************
* EXPORTED DATA                  g[k|p|kp|pk|kpk]phStbDbg_<Word>+ *
*******************************************************************/

/*******************************************************************************
* FUNCTION IMPLEMENTATION  <Module>[_<Word>+] for static functions             *
*                          tm[<layer>]<Module>[_<Word>+] for exported functions*
********************************************************************************/

int main(int argc, char **argv)
{
    unsigned long iterations, i;

    if (argc != 2)
    {
        fprintf(stderr, "Usage: %s NUM_ITERATIONS\n", argv[0]);
        fprintf(stderr, "Runs DBG_INIT and DBG_REGISTER_MODULE NUM_ITERATIONS times\n");
        return 1;
    }

    iterations = atol(argv[1]);
    fprintf(stderr, "You've chosen %lu interations of DBG_INIT / DBG_REGISTER_MODULE\n",
            iterations);

    system("cat /proc/meminfo");
    for (i = 0; i < iterations; i++)
    {
        DBG_INIT();
        DBG_REGISTER_MODULE(gDbgUnit, "StbDbg_UnitTest2");
        DBG_SET_FLAGS(gDbgUnit, DBG_ALL_FLAGS);
    }
    printf("\n%lu iterations done\n\n", iterations);
    system("cat /proc/meminfo");
    printf("\n%s DONE\n", argv[0]);

    return 0;
}

/******************************************************************************/
