#ifndef PHSTBIPC_SERVICES_H    /* Multi-include protection.   <FILENAME>_H */
#define PHSTBIPC_SERVICES_H


/*****************************************************************************/
/* Copyright (c) 2009 NXP B.V.                                               */
/*                                                                           */
/* This library is free software; you can redistribute it and/or             */
/* modify it under the terms of the GNU Lesser General Public                */
/* License as published by the Free Software Foundation, using               */
/* version 2.1 of the License.                                               */
/*                                                                           */
/* This library is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU          */
/* Lesser General Public License for more details.                           */
/*                                                                           */
/* You should have received a copy of the GNU Lesser General Public          */
/* License along with this library; if not, write to the Free Software       */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   */
/*                                                                           */
/*****************************************************************************/
/**
    @file   phStbIpc_Services.h
    @brief  Service definitions for phStbIpc.

@b  Component:  phStbIpc

    A server address is a string containing three parts: "PROTOCOL://HOST:PORT".

    PROTOCOL is "tcpip".

    For protocol "tcpip", HOST can be:
    - "mips1" to indicate the control/main MIPS
    - "mips2" to indicate the dsp/audio MIPS
    - "AAA.BBB.CCC.DDD" to indicate a specific IP address
    - "127.0.0.1" to indicate the current host.

    PORT is a numeric port ID, e.g. "1234".

    Example server addresses:
    - "tcpip://mips2:1234"
    - "tcpip://127.0.0.1:1234"
    - "tcpip://192.168.0.2:1234"
*//*

Rev Date        Author      Comments
--------------------------------------------------------------------------------
  1 20080408    batelaan    Initial
  2 20090121    batelaan    Update license header
--------------------------------------------------------------------------------
    For consistency and standardisation retain the Section Separators.
*/


/*******************
* INCLUDE FILES    *
********************/

#ifndef PHSTBIPC_H
#include <phStbIpc.h>
#endif

/**************************************************************************
* EXPORTED MACROS                          PH_STBIPC_<_WORD>+             *
***************************************************************************/

/** Server address used by phStbHSServer and phStbHSClient.
    @see phStbHSServer.h
    @see phStbHSClient.h
   */
#define PH_STBIPC_HSSERVER_SERVER_ADDRESS  "tcpip://mips1:4323"

/** Server address used by phdspServer and phStbDspClient.
    @see phdspServer.h
    @see phStbDspClient.h
*/
#define PH_STBIPC_DSP_SERVER_ADDRESS       "tcpip://mips2:8765"
#define PH_STBIPC_DSP_LOCAL_SERVER_ADDRESS "tcpip://127.0.0.1:8765"

/** Test server address used by phStbDspClient.
    @see phStbDspClient.h
*/
#define PH_STBIPC_DSP_TEST_SERVER_ADDRESS "tcpip://mips2:8999"

/** Server address used by phStbDspClient.
    @see phStbDspClient.h
*/
#define PH_STBIPC_UNITTEST_SERVER_ADDRESS "tcpip://mips2:12345"

/** Server address used by exStbTop on the DSP.
    @see exStbTop.c
*/
#define PH_STBIPC_DSP_EXSTBTOP_SERVER_ADDRESS "tcpip://mips2:12346"

/*********************************************************************
* EXPORTED TYPEDEFS                      [p]phStbIpc_<Word>+_t       *
**********************************************************************/

/**************************************************************************
* EXPORTED DATA                     g[k|p|kp|pk|kpk]phStbIpc_<Word>+      *
***************************************************************************/

/****************************************************************************
* EXPORTED FUNCTION PROTOTYPES                    phStbIpc[_<Word>+]        *
*****************************************************************************/

#ifdef __cplusplus
extern "C"
{
#endif

#ifdef __cplusplus
}
#endif

/******************************************************************************/
#ifdef __cplusplus
}
#endif

#endif /* PHSTBIPC_SERVICES_H      Do not add any thing below this line */
