/*****************************************************************************/
/* Copyright (c) 2008 NXP B.V.                                               */
/*                                                                           */
/* This program is free software; you can redistribute it and/or modify      */
/* it under the terms of the GNU General Public License as published by      */
/* the Free Software Foundation, using version 2 of the License.             */
/*                                                                           */
/* This program is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the              */
/* GNU General Public License for more details.                              */
/*                                                                           */
/* You should have received a copy of the GNU General Public License         */
/* along with this program; if not, write to the Free Software               */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307       */
/* USA.                                                                      */
/*                                                                           */
/*****************************************************************************/
/**
    @file   phStbNetDev_Lock.c
    @brief  Lock functionality for phStbNetDev component.

@b  Component:  phStbNetDev.

    Two-process or CPU mutual exclusion algorithm.

    Based on Peterson, G.L. "Myths about the mutual exclusion problem",
    Information Processing Letters 12, 3 (13 June 1981), 115-116.

    @see http://vijucat.wordpress.com/2006/01/29/petersons-2-process-mutual-exclusion-algorithm/
    @see http://www.teamten.com/lawrence/242.paper/242.paper.pdf
*//*

Rev Date       Author      Comments
--------------------------------------------------------------------------------
  1 20061207   batelaan    Initial
  2 20061218   batelaan    Add ref to source for locking algorithm.
  3 20070501   batelaan    Only do inter-cpu locking.
                           Use of mutex is done elsewhere.
  4 20070611   steel       fix cache race condition
  5 20080422   batelaan    Update file header
--------------------------------------------------------------------------------
    For consistency and standardisation retain the Section Separators.
*/

#include <phStbNetDev_Internal.h>

/***********************************************
* LOCAL MACROS                                 *
* recommendation only: <MODULE><_WORD>+        *
************************************************/

/******************************************************************
* FUNCTION PROTOTYPES                    [p]ph<Module>_<Word>+    *
*******************************************************************/

/******************************************************************
* LOCAL TYPEDEFS                                                  *
* recommendation only:                   [p]ph<Module>_<Word>+_t  *
*******************************************************************/

/*******************************************************************
* EXPORTED DATA                 g[k|p|kp|pk|kpk]ph<Module>_<Word>+ *
********************************************************************/

/*******************************************************************
* LOCAL DATA                                                       *
********************************************************************/

/*******************************************************************************
* FUNCTION IMPLEMENTATION  <Module>[_<Word>+] for static functions             *
*                          tm[<layer>]<Module>[_<Word>+] for exported functions*
********************************************************************************/

/******************************************************************************/

void phStbNetDev_LockInitUnlocked(volatile phStbNetDev_Lock_t *lock)
{
    lock->interested[0] = 0;
    lock->interested[1] = 0;
    lock->guard         = 0;
}

/******************************************************************************/

void phStbNetDev_Lock(volatile phStbNetDev_Lock_t *lock, Int processId)
{
    Int otherId = 1 - processId;

    lock->interested[processId] = 1;
    lock->turn = processId;
    while ((lock->turn == processId) && (lock->interested[otherId] == 1))
    {
        lock->guard++;
    }
}

/******************************************************************************/

void phStbNetDev_Unlock(volatile phStbNetDev_Lock_t *lock, Int processId)
{
    lock->interested[processId] = 0;
    lock->guard++;
}

/******************************************************************************/
