/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright (C) 2002,2003 Florian Schirmer <jolt@tuxbox.org>
 *
 * Copyright (C) 2007 NXP B.V.
 * All Rights Reserved.
 *
 * NXP based DVB adapter driver.
 * Based on the bt8xx driver by Florian Schirmer <jolt@tuxbox.org>
 *
*//*

Rev Date        Author        Comments
--------------------------------------------------------------------------------
001 20080306    laird         Import from 2.6.19.1 overlay tree.
002 20080306    laird         Fix for linux-2.6.24_nxp build.
003 20080526    bmagnier      Import some updates from the 2.6.19.1 from Ch.Grosse
                              (split dvb & dvb_frontend)
--------------------------------------------------------------------------------
    For consistency and standardisation retain the Section Separators.
*/

#include <linux/module.h>
#include <linux/init.h>

#include "phStbDemux_common.h"
#include "phStbDemux_dvb.h"
#include "phStbDemux_frontend.h"

static int __init dvb_phStb_init(void)
{
   int retval = 0;

   phStbFrontend_init();
   
   printk ("%s (%s-%s)\n", THIS_MODULE_DESCRIPTION, __DATE__, __TIME__);
   return retval;
}

static void __exit dvb_phStb_exit(void)
{
   phStbFrontend_exit();
   printk("Shutting down %s\n", THIS_MODULE_DESCRIPTION);
}

module_init(dvb_phStb_init);
module_exit(dvb_phStb_exit);

MODULE_DESCRIPTION(THIS_MODULE_DESCRIPTION);
MODULE_AUTHOR("Chris Steel");
MODULE_LICENSE("GPL");

module_param(debug, int, 0644);
MODULE_PARM_DESC(debug, " Turn on (=1) debugging. Default off (=0).");

module_param(gMin_Freq, long, 0444);
MODULE_PARM_DESC(gMin_Freq, " FrontEnd's Min. Freq. Default=174000000");

module_param(gMax_Freq, long, 0444);
MODULE_PARM_DESC(gMax_Freq, " FrontEnd's Max. Freq. Default=862000000");

module_param(fe_model_name, charp, S_IRUSR | S_IRGRP | S_IROTH);
MODULE_PARM_DESC(fe_model_name, " FrontEnd/tuner type. Build settings: TU1216, TDA8275A ...");

